@@ -19,11 +19,11 @@ class DFlowFMWriter:
 
     # versioning info
     version = { 'number'        : delft3dfmpy.__version__,
-                'date'          : datetime.datetime.strftime(datetime.datetime.today(),'%Y-%m-%d %H:%M:%S'),
-                'dfm_version'   : '1.2.105.67228M',
-                'dimr_version'  : '2.13.02.67836',
-                'suite_version' : '(Beta) 0.9.6.51435'} 
-   
+                'date'          :  datetime.datetime.strftime(datetime.datetime.utcnow(),'%Y-%m-%dT%H:%M:%S.%fZ'),
+                'dfm_version'   : 'Deltares, D-Flow FM Version 1.2.124.69571M',
+                'dimr_version'  : 'Deltares, DIMR_EXE Version 2.00.00.69571M (Win64)',
+                'suite_version' : 'D-HYDRO Suite 1D2D (1.0.0.53506),'} 
+     
     def __init__(self, dflowfmmodel, output_dir, name):
         self.dflowfmmodel = dflowfmmodel
         self.output_dir = os.path.join(output_dir, 'fm')
@@ -67,10 +67,10 @@ class DFlowFMWriter:
         self.writeMdFiles()
         # Roughness files
         self.write_roughness()
-        # Cross section definitions
-        self.write_crosssection_definitions()
         # Cross section locations
         self.write_crosssection_locations()
+        # Cross section definitions
+        self.write_crosssection_definitions()        
         # Structures (weirs, pumps, culverts)
         self.write_structures()
         # Initial conditions
@@ -82,6 +82,8 @@ class DFlowFMWriter:
         if hasattr(self, 'run_dimrpad'):
             # Files for running DIMR
             self.write_dimrfiles()
+        # Storage nodes
+        self.write_storagenodes()
         
         # Change mdu parameters
         for parameter, value in self.mdu_parameters.items():
@@ -140,6 +142,18 @@ class DFlowFMWriter:
 
         self.mdu_parameters['frictFile'] = ';'.join(roughnessfiles)
 
+    def write_storagenodes(self):
+        filename = 'storagenodes.ini'
+        filepath = os.path.join(self.output_dir, filename)
+        if any(self.dflowfmmodel.storagenodes.storagenodes):
+        
+            with open(filepath, 'w') as f:
+                self._write_header(f, filetype='storageNodes', fileversion=2.00)            
+                for _, dct in self.dflowfmmodel.storagenodes.storagenodes.items():
+                    self._write_dict(f, dct=dct, header='StorageNode')
+
+            self.mdu_parameters['StorageNodeFile'] = filename
+
     def write_crosssection_locations(self):
         # Write cross section locations
         with open(os.path.join(self.output_dir, 'cross_section_locations.ini'), 'w') as f:
@@ -153,13 +167,22 @@ class DFlowFMWriter:
                 # Find branches without profile
                 no_css = self.dflowfmmodel.crosssections.get_branches_without_crosssection()
                 branches = (self.dflowfmmodel.network.branches.loc[no_css, 'geometry'].length / 2).to_dict()
-                for branchid, chainage in branches.items():
+                ibranch=0
+                for branchid, chainage in branches.items():                    
+                    ibranch+=1
+                    self.dflowfmmodel.crosssections.crosssection_def['default_'+str(ibranch)] = {'id':'default_'+str(ibranch),
+                             'type': self.dflowfmmodel.crosssections.crosssection_def['default']['type'],
+                             'thalweg': self.dflowfmmodel.crosssections.crosssection_def['default']['thalweg'],
+                             'height': self.dflowfmmodel.crosssections.crosssection_def['default']['height'],
+                             'width': self.dflowfmmodel.crosssections.crosssection_def['default']['width'],
+                             'closed': self.dflowfmmodel.crosssections.crosssection_def['default']['closed'],
+                             'frictionId': self.dflowfmmodel.crosssections.crosssection_def['default']['frictionId']}
                     dct = {
                         'id': f'{branchid}_{chainage:.1f}',
                         'branchid': branchid,
                         'chainage': chainage,
                         'shift': self.dflowfmmodel.crosssections.default_definition_shift,
-                        'definitionId': self.dflowfmmodel.crosssections.default_definition
+                        'definitionId': self.dflowfmmodel.crosssections.crosssection_def['default_'+str(ibranch)]['id']
                     }
                     # Write the cross section item
                     self._write_dict(f, dct=dct, header='CrossSection')
@@ -188,8 +211,13 @@ class DFlowFMWriter:
         filepath = os.path.join(self.output_dir, 'structure.ini')
         with open(filepath, 'w') as f:
             # write header
-            self._write_header(f, filetype='structure', fileversion=2.00)
+            self._write_header(f, filetype='structure', fileversion=3.00)
 
+            # General structures
+            if any(self.dflowfmmodel.structures.generalstructures):
+                for _, dct in self.dflowfmmodel.structures.generalstructures.items():
+                    self._write_dict(f, dct=dct, header='Structure')
+            
             # Culverts
             if any(self.dflowfmmodel.structures.culverts):
                 for _, dct in self.dflowfmmodel.structures.culverts.items():
@@ -224,16 +252,27 @@ class DFlowFMWriter:
             if any(self.dflowfmmodel.structures.compounds):
                 for _, dct in self.dflowfmmodel.structures.compounds.items():
                     self._write_dict(f, dct=dct, header='structure')
-
+        
+        refd = str(self.dflowfmmodel.mdu_parameters["refdate"])
+        
         # Write structure time series
-        for bcnd in self.dflowfmmodel.external_forcings.structures.itertuples():
-            # Write time series for structure
-            filename = f"{bcnd.type}_{bcnd.id}.tim"
+        for bcnd in self.dflowfmmodel.external_forcings.structures.iterrows():
+            
+            # write structure time series to bc file            
+            with open(os.path.join(self.output_dir, 'boundaries.bc'), 'a') as f:
+                f.write(f'\n[Forcing]\n'
+                    f'name       = {bcnd[1]["id"]}\n'
+                    f'function   = timeseries\n'
+                    f'timeInterpolation = linear\n'
+                    f'quantity   = time\n'
+                    f'unit       = {bcnd[1]["time_unit"]}\n'
+                    f'quantity   = {bcnd[1]["parameter"]}\n'
+                    f'unit       = {bcnd[1]["value_unit"]}\n'
+                   )
             # Get time series as data
-            data = list(zip(bcnd.time, bcnd.value))
+            data = list(zip(bcnd[1].time, bcnd[1].value))
             # Write file
-            write_fm_file(file=os.path.join(self.output_dir, filename), data=data)
-
+            write_fm_file(file=os.path.join(self.output_dir,  'boundaries.bc'), data=data, mode='a')
 
     def write_laterals(self):
         """
@@ -245,17 +284,13 @@ class DFlowFMWriter:
 
         for name, dct in self.dflowfmmodel.external_forcings.laterals.items():
 
-            if 'timeseries' in dct:
-                # Determine starttime from mdu parameters
-                starttime = datetime.datetime.strptime(str(self.mdu_parameters['refdate']), '%Y%m%d')
-                starttime += datetime.timedelta(seconds=int(self.mdu_parameters['tstart']))
-
+            if 'timeseries' in dct:               
                 # Write time series - this should probably be written to a .bc file
                 series = dct['timeseries'].copy()
-                series.index -= starttime
-                data = np.c_[series.index.total_seconds().values / 60, series.values]
-
-                refd = str(self.dflowfmmodel.mdu_parameters["refdate"])
+                
+                starttime = datetime.datetime.strftime(series.index[0], '%Y%m%d%H')
+                series.index -= series.index[0]
+                data = np.c_[series.index.total_seconds().values / 60, series.values]               
 
                  # and write data to the bc file
                 with open(os.path.join(self.output_dir, 'boundaries.bc'), 'a') as f:
@@ -264,7 +299,7 @@ class DFlowFMWriter:
                         f'function   = timeseries\n'
                         f'timeInterpolation = linear\n'
                         f'quantity   = time\n'
-                        f'unit       = minutes since {refd[0:4]}-{refd[4:6]}-{refd[6:9]} 00:00:00\n'
+                        f'unit       = minutes since {starttime[0:4]}-{starttime[4:6]}-{starttime[6:8]} {starttime[8:10]}:00:00\n'
                         f'quantity   = lateral_discharge\n'
                         f'unit       = m3/s\n'
                        )
@@ -344,16 +379,16 @@ class DFlowFMWriter:
                             f'function   = timeseries\n'
                             f'timeInterpolation = linear\n'
                             f'quantity   = time\n'
-                            f'unit       = minutes since {refd[0:4]}-{refd[4:6]}-{refd[6:9]} 00:00:00\n'
+                            f'unit       = {bc["time_unit"]}\n'
                             f'quantity   = {bc["bctype"]}\n'
-                            f'unit       = {bc["unit"]}\n'
+                            f'unit       = {bc["value_unit"]}\n'
                            )
                     if headerType=='constant':
                         f.write(f'\n[Forcing]\n'
                             f'name       = {bc["nodeid"]}\n'
                             f'function   = constant\n'
                             f'quantity   = {bc["bctype"]}\n'
-                            f'unit       = {bc["unit"]}\n'
+                            f'unit       = {bc["value_unit"]}\n'
                             f'{data}\n'
                            )
                 if headerType=='timeseries':
@@ -366,7 +401,7 @@ class DFlowFMWriter:
         """
 
         # Create folder for initial conditions if it does not exist yet
-        initcondfolder = 'initialconditions'
+        initcondfolder = '.'
 
         # initial files are not in the ext-file in the new format...
         self.dflowfmmodel.mdu_parameters['IniFieldFile'] = initcondfolder+'/initialFields.ini'
@@ -384,7 +419,9 @@ class DFlowFMWriter:
                 # no polygons defined - global definition
                 with open(os.path.join(initcondpath,'initialWaterLevel.ini'),'w') as f:
                     self._write_header(f, '1dField',2.00)
-                    dct = {'quantity': 'waterLevel','unit': 'm+NAP','value': self.dflowfmmodel.external_forcings.initial_waterlevel_polygons.waterlevel[0]}
+                    dct = {'quantity': 'waterLevel','unit': 'm+NAP',
+                           'value': self.dflowfmmodel.external_forcings.initial_waterlevel_polygons.waterlevel[0], 
+                           'locationType': self.dflowfmmodel.external_forcings.initial_waterlevel_polygons.locationType[0]}
                     self._write_dict(f, dct, 'Global')
 
                 with open(os.path.join(initcondpath,'initialFields.ini'),'a') as f:
@@ -403,9 +440,10 @@ class DFlowFMWriter:
                     with open(os.path.join(initcondpath,'initialFields.ini'),'a') as f:
                         dct  = {'quantity': 'waterLevel',
                                 'dataFileType': 'polygon',
-                                'dataFile': f'{row.Index}.pol',
+                                'dataFile': initcondfolder+f'/{row.Index}.pol',
                                 'interpolationMethod': 'constant',
-                                'value': f'{row.waterlevel}'
+                                'value': f'{row.waterlevel}',
+                                'locationType': f'{row.locationType}'
                                 }
                         self._write_dict(f,dct,'Initial')
 
@@ -415,7 +453,9 @@ class DFlowFMWriter:
                 # no polygons defined - global definition
                 with open(os.path.join(initcondpath,'initialWaterDepth.ini'),'w') as f:
                     self._write_header(f, '1dField',2.00)
-                    dct = {'quantity': 'waterDepth','unit': 'm','value': self.dflowfmmodel.external_forcings.initial_waterdepth_polygons.waterdepth[0]}
+                    dct = {'quantity': 'waterDepth','unit': 'm',
+                           'value': self.dflowfmmodel.external_forcings.initial_waterdepth_polygons.waterdepth[0],
+                           'locationType': self.dflowfmmodel.external_forcings.initial_waterdepth_polygons.locationType[0]}
                     self._write_dict(f, dct, 'Global')
 
                 with open(os.path.join(initcondpath,'initialFields.ini'),'a') as f:
@@ -436,9 +476,10 @@ class DFlowFMWriter:
                         dct  = {'quantity': 'waterDepth',
                                 'unit': 'm',
                                 'dataFileType': 'polygon',
-                                'dataFile': f'{row.Index}.pol',
+                                'dataFile': initcondfolder+f'/{row.Index}.pol',
                                 'interpolationMethod': 'constant',
-                                'value': f'{row.waterdepth}'
+                                'value': f'{row.waterdepth}',
+                                'locationType': f'{row.locationType}'
                                 }
                         self._write_dict(f,dct,'Initial')
 
@@ -452,7 +493,14 @@ class DFlowFMWriter:
 
         network = self.dflowfmmodel.network
         crosssections = self.dflowfmmodel.crosssections
-        structures = self.dflowfmmodel.structures.as_dataframe(weirs=True, pumps=True, bridges=True, culverts=True, uweirs=True, orifices=True, compounds=True)
+        structures = self.dflowfmmodel.structures.as_dataframe(generalstructures=True,
+                                                               weirs=True,
+                                                               pumps=True,
+                                                               bridges=True,
+                                                               culverts=True,
+                                                               uweirs=True,
+                                                               orifices=True,
+                                                               compounds=True)
 
         for name, css in crosssections.crosssection_loc.items():
             sch_branch = network.schematised.at[css['branchid'], 'geometry']
@@ -485,7 +533,12 @@ class DFlowFMWriter:
                 # Get offset on schematised branch
                 offset = sch_branch.project(geo_branch.interpolate(structure.chainage))
 
-                if structure.structype == 'pump':
+                if structure.structype == 'generalstructure':
+                    # Get circle coordinates
+                    coords.append(sch_branch.interpolate(offset).buffer(20.0).simplify(0.5).exterior.coords[:])
+                    names.append(structure.Index)
+
+                elif structure.structype == 'pump':
                     # Get circle coordinates
                     coords.append(sch_branch.interpolate(offset).buffer(20.0).simplify(0.5).exterior.coords[:])
                     names.append(structure.Index)
@@ -556,9 +609,8 @@ class DFlowFMWriter:
                 self._write_header(f, filetype='obsPoints', fileversion=2.00)
                 self.dflowfmmodel.observation_points.drop('geometry', axis=1, inplace=True)
                 if self.dflowfmmodel.observation_points['locationType'].str.count("1d").sum()>0:
-                    obs1d = self.dflowfmmodel.observation_points[self.dflowfmmodel.observation_points['locationType']=='1d']
-                    if self.dflowfmmodel.observation_points['locationType'].str.count("2d").sum()>0:
-                        obs1d.drop(['x','y'], axis=1, inplace=True)
+                    obs1d = self.dflowfmmodel.observation_points[self.dflowfmmodel.observation_points['locationType']=='1d']                    
+                    obs1d.drop(['x','y'], axis=1, inplace=True)
                     obs1ddct = obs1d.to_dict(orient='row')
                     for dct in obs1ddct:
                         self._write_dict(f, dct=dct, header='ObservationPoint')
@@ -656,3 +708,4 @@ def write_fm_file(file, data, names=None, mode='w'):
     with open(file, mode) as f:
         f.write(string)
 
+
