@@ -18,7 +18,7 @@ logger = logging.getLogger(__name__)
 class DFlowRRWriter:
     """Writer for RR files"""
 
-    def __init__(self, rrmodel, output_dir, name):
+    def __init__(self, rrmodel, output_dir, name, wwtp=None):
         self.rrmodel = rrmodel
         # self.geometries = parent.geometries
         # self.boundary_conditions = parent.boundary_conditions
@@ -35,6 +35,11 @@ class DFlowRRWriter:
         self.precip_df  = {}
         self.run_dimrpad = rrmodel.dimr_path
         
+        if wwtp is None:
+            self.wwtp = Point((1e5,5e5))
+        else:
+            self.wwtp  = Point((wwtp[0],wwtp[1]))
+
     def write_all(self):  # write all RR files
         """
         Wrapper method to write all components
@@ -101,7 +106,10 @@ class DFlowRRWriter:
                 for _, dct in self.rrmodel.paved.pav_nodes.items():
                     if float(dct['ar']) > 0.0:                    
                         f.write('NODE id \''+dct['id']+'\' nm \''+dct['id']+'\' ri \'-1\' mt 1 \'1\' nt 43 ObID \'3B_PAVED\' px '+dct['px']+' py '+dct['py']+' node\n')
-                  
+
+                f.write('NODE id \'WWTP\' nm \'WWTP\' ri \'-1\' mt 1 \'14\' nt 56 ObID \'3B_WWTP\' px '+str(self.wwtp.coords[0][0])+' py '+str(self.wwtp.coords[0][1])+' node\n')
+                f.write('NODE id \'WWTP_BND\' nm \'WWTP_BND\' ri \'-1\' mt 1 \'6\' nt 47 ObID \'3B_BOUNDARY\' px '+str(self.wwtp.coords[0][0]+50.)+' py '+str(self.wwtp.coords[0][1]+50.)+' node\n')
+
             # Greenhouse nodes
             if any(self.rrmodel.greenhouse.gh_nodes):
                 for _, dct in self.rrmodel.greenhouse.gh_nodes.items():
@@ -115,7 +123,7 @@ class DFlowRRWriter:
                         f.write('NODE id \''+dct['id']+'\' nm \''+dct['id']+'\' ri \'-1\' mt 1 \'21\' nt 46 ObID \'OW_PRECIP\' px '+dct['px']+' py '+dct['py']+' node\n')
                       
             # Boundary_nodes
-            if any(self.rrmodel.external_forcings.boundary_nodes):
+            if any(self.rrmodel.external_forcings.boundary_nodes):               
                 for _, dct in self.rrmodel.external_forcings.boundary_nodes.items():
                     f.write('NODE id \''+dct['id']+'\' nm \''+dct['id']+'\' ri \'-1\' mt 1 \'6\' nt 78 ObID \'SBK_SBK-3B-NODE\' px '+dct['px']+' py '+dct['py']+' node\n')
                     
@@ -127,32 +135,39 @@ class DFlowRRWriter:
                 for _, dct in self.rrmodel.unpaved.unp_nodes.items():
                     if np.sum([float(d) for d in dct['ar'].split(' ')]) > 0.0:                    
                         cnt += 1
-                        f.write('BRCH id \''+str(cnt)+'\' ri \'-1\' mt 1 \'0\' bt 17 ObID \'3B_LINK\' bn \''+dct['id']+'\' en \''+dct['boundary_node']+'\' brch\n')
+                        f.write('BRCH id \'link_'+str(cnt)+'\' ri \'-1\' mt 1 \'0\' bt 17 ObID \'3B_LINK\' bn \''+dct['id']+'\' en \''+dct['boundary_node']+'\' brch\n')
             if any(self.rrmodel.paved.pav_nodes):                
                 for _, dct in self.rrmodel.paved.pav_nodes.items():
                     if float(dct['ar']) > 0.0:                    
                         cnt += 1
-                        f.write('BRCH id \''+str(cnt)+'\' ri \'-1\' mt 1 \'0\' bt 17 ObID \'3B_LINK\' bn \''+dct['id']+'\' en \''+dct['boundary_node']+'\' brch\n')
+                        f.write('BRCH id \'link_'+str(cnt)+'\' ri \'-1\' mt 1 \'0\' bt 17 ObID \'3B_LINK\' bn \''+dct['id']+'\' en \''+dct['boundary_node']+'\' brch\n')
+                        cnt += 1
+                        f.write("BRCH id \'link_"+str(cnt)+"\' ri \'-1\' mt 1 \'1\' bt 18 ObID \'3B_LINK_RWZI\' bn \'"+dct['id']+"\' en \'WWTP\' brch\n")
+                cnt += 1                             
+                f.write('BRCH id \'link_'+str(cnt)+'\' ri \'-1\' mt 1 \'0\' bt 17 ObID \'3B_LINK\' bn \'WWTP\' en \'WWTP_BND\' brch\n')
+
             if any(self.rrmodel.greenhouse.gh_nodes):                
                 for _, dct in self.rrmodel.greenhouse.gh_nodes.items():
                     if float(dct['ar']) > 0.0:                    
                         cnt += 1
-                        f.write('BRCH id \''+str(cnt)+'\' ri \'-1\' mt 1 \'0\' bt 17 ObID \'3B_LINK\' bn \''+dct['id']+'\' en \''+dct['boundary_node']+'\' brch\n')
+                        f.write('BRCH id \'link_'+str(cnt)+'\' ri \'-1\' mt 1 \'0\' bt 17 ObID \'3B_LINK\' bn \''+dct['id']+'\' en \''+dct['boundary_node']+'\' brch\n')
             if any(self.rrmodel.openwater.ow_nodes):                
                 for _, dct in self.rrmodel.openwater.ow_nodes.items():
                     if float(dct['ar']) > 0.0:                    
                         cnt += 1
-                        f.write('BRCH id \''+str(cnt)+'\' ri \'-1\' mt 1 \'0\' bt 17 ObID \'3B_LINK\' bn \''+dct['id']+'\' en \''+dct['boundary_node']+'\' brch\n')
+                        f.write('BRCH id \'link_'+str(cnt)+'\' ri \'-1\' mt 1 \'0\' bt 17 ObID \'3B_LINK\' bn \''+dct['id']+'\' en \''+dct['boundary_node']+'\' brch\n')
 
         # bound3b.3b        
         filepath = os.path.join(self.output_dir, 'Bound3B.3B')
         with open(filepath, 'w') as f:
             for _, dct in self.rrmodel.external_forcings.boundary_nodes.items():                
                 f.write('BOUN id \''+dct['id']+'\' bl 2 \'0\' is 0 boun\n')
+            if any(self.rrmodel.paved.pav_nodes):    
+                f.write('BOUN id \'WWTP_BND\' bl 0 -3 is 0 boun\n')        
                 
         # BoundaryConditions.bc
         filepath = os.path.join(self.output_dir, 'BoundaryConditions.bc')
-        header = {'majorVersion':'1', 'minorVersion':'0', 'fileType':'boundConds'}
+        header = {'fileVersion': '1.01', 'fileType':'boundConds'}
         with open(filepath, 'w') as f:                        
             self._write_dict(f, header, 'General','\n')            
             for _, dct in self.rrmodel.external_forcings.boundary_nodes.items():                
@@ -173,7 +188,7 @@ class DFlowRRWriter:
             with open(filepath, 'w') as f:
                 for _, dct in self.rrmodel.unpaved.unp_nodes.items():
                     if np.sum([float(d) for d in dct['ar'].split(' ')]) > 0.0:                    
-                        f.write('UNPV id \''+dct['id']+'\' na '+dct['na']+' gw '+dct['ga']+' ar '+dct['ar']+' lv '+dct['lv']+' co '+dct['co']+' su '+dct['su']+' sd \'sto_'+dct['id']+'\' ic \'inf_'+dct['id']+'\' bt '+dct['bt']+' ed \''+dct['ed']+'\' sp \''+dct['sp']+'\' ig 0 '+dct['ig']+' mg '+dct['mg']+' gl '+dct['gl']+' is '+dct['is']+' ms \''+dct['ms']+'\' unpv\n')                    
+                        f.write('UNPV id \''+dct['id']+'\' na '+dct['na']+' ga '+dct['ga']+' ar '+dct['ar']+' lv '+dct['lv']+' co '+dct['co']+' su '+dct['su']+' sd \'sto_'+dct['id']+'\' ic \'inf_'+dct['id']+'\' bt '+dct['bt']+' ed \''+dct['ed']+'\' sp \''+dct['sp']+'\' ig 0 '+dct['ig']+' mg '+dct['mg']+' gl '+dct['gl']+' is '+dct['is']+' ms \''+dct['ms']+'\' unpv\n')                    
                 
             filepath = os.path.join(self.output_dir, 'UNPAVED.STO')
             with open(filepath, 'w') as f:            
@@ -189,7 +204,8 @@ class DFlowRRWriter:
     
             filepath = os.path.join(self.output_dir, 'UNPAVED.ALF')
             with open(filepath, 'w') as f:            
-                for _, dct in self.rrmodel.unpaved.ernst_defs.items():                    
+                for _, dct in self.rrmodel.unpaved.ernst_defs.items():        
+                    if np.sum([float(d) for d in self.rrmodel.unpaved.unp_nodes[_]['ar'].split(' ')]) > 0.0:
                         f.write('ERNS id \''+dct['id']+'\' nm \''+dct['id']+'\' cvi '+dct['cvi']+' cvo 0 '+dct['cvo']+' cvs '+dct['cvs']+' lv '+dct['lv']+' erns\n')                    
                                                 
             filepath = os.path.join(self.output_dir, 'UNPAVED.SEP')
@@ -216,7 +232,7 @@ class DFlowRRWriter:
             with open(filepath, 'w') as f:          
                 for _, dct in self.rrmodel.paved.pav_nodes.items():                    
                     if float(dct['ar']) > 0.0:                    
-                        f.write('PAVE id \''+dct['id']+'\' ar '+dct['ar']+' lv '+dct['lv']+' sd \'sto_'+dct['id']+'\' ss 0 qc 0 '+dct['qc']+' 0 qo 1 0 ms \''+dct['ms']+'\' aaf 1 is '+dct['is']+' np '+dct['np']+' dw \'Def_DWA\' ro '+dct['ro']+' ru '+dct['ru']+' qh \'\' pave\n')
+                        f.write('PAVE id \''+dct['id']+'\' ar '+dct['ar']+' lv '+dct['lv']+' sd \'sto_'+dct['id']+'\' ss 0 qc 0 '+dct['qc']+' 0 qo 1 0 ms \''+dct['ms']+'\' is '+dct['is']+' np '+dct['np']+' dw \'Def_DWA\' ro '+dct['ro']+' ru '+dct['ru']+' qh \'\' pave\n')
                                                                                                                                                                                                            
             filepath = os.path.join(self.output_dir, 'PAVED.STO')                       
             with open(filepath, 'w') as f:           
@@ -227,7 +243,11 @@ class DFlowRRWriter:
             filepath = os.path.join(self.output_dir, 'PAVED.DWA')             
             with open(filepath, 'w') as f:                           
                 f.write('DWA id \'Def_DWA\' nm \'Def_DWA\' do 1 wc 0 wd 0 wh 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 dwa\n')
-                    
+
+            filepath = os.path.join(self.output_dir, 'WWTP.3B')             
+            with open(filepath, 'w') as f:                           
+                f.write('WWTP id \'WWTP\' tb 0  wwtp\n')                
+
     def write_greenhouse(self):
         """
         Method to write all files associated with greenhouse nodes: GREENHSE.3B, GREENHSE.RF and GREENHSE.SIL. The latter contains only one definition.
@@ -377,8 +397,8 @@ class DFlowRRWriter:
                  f.write('pause\n')
 
         # coupling XML
-        DFM_comp_name = 'CoupledTest1_DFM'
-        RR_comp_name = 'CoupledTest1_RR'
+        DFM_comp_name = 'DFM'
+        RR_comp_name = 'RR'
         with open(os.path.join(self.output_dir, '../dimr_config.xml'),'w') as f:       
                  f.write('<?xml version="1.0" encoding="utf-8" standalone="yes"?>\n')
                  f.write('<dimrConfig xmlns="http://schemas.deltares.nl/dimr" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://schemas.deltares.nl/dimr http://content.oss.deltares.nl/schemas/dimr-1.2.xsd">\n')
@@ -416,8 +436,6 @@ class DFlowRRWriter:
                  f.write('\t\t<targetComponent>'+RR_comp_name+'</targetComponent>\n')		     
                  for i in self.rrmodel.external_forcings.boundary_nodes.items():
                      f.write('\t\t\t<item>\n')                     
-                     # FOr now use one observation ponit- until water levels can be read from other nodes
-                     #f.write('\t\t\t\t<sourceName>observations/'+str(i[0])+'/water_level</sourceName>\n')
                      f.write('\t\t\t\t<sourceName>laterals/'+str(i[0])+'/water_level</sourceName>\n')
                      f.write('\t\t\t\t<targetName>catchments/'+str(i[0])+'/water_level</targetName>\n')
                      f.write('\t\t\t</item>\n')                                  
